@@ -15,10 +15,12 @@
 package main
 
 import (
+	"context"
 	"crypto/tls"
 	"log"
 	"strconv"
 	"strings"
+	"sync"
 	"time"
 
 	irc "github.com/fluffle/goirc/client"
@@ -37,7 +39,7 @@ const (
 var (
 	ircConnectedGauge = promauto.NewGauge(prometheus.GaugeOpts{
 		Name: "irc_connected",
-		Help: "Wether the IRC connection is established",
+		Help: "Whether the IRC connection is established",
 	})
 	ircSentMsgs = promauto.NewCounterVec(prometheus.CounterOpts{
 		Name: "irc_sent_msgs",
@@ -63,12 +65,13 @@ type ChannelState struct {
 type IRCNotifier struct {
 	// Nick stores the nickname specified in the config, because irc.Client
 	// might change its copy.
-	Nick           string
-	NickPassword   string
-	Client         *irc.Conn
-	StopRunning    chan bool
-	StoppedRunning chan bool
-	AlertMsgs      chan AlertMsg
+	Nick         string
+	NickPassword string
+	Client       *irc.Conn
+	AlertMsgs    chan AlertMsg
+
+	ctx    context.Context
+	stopWg *sync.WaitGroup
 
 	// irc.Conn has a Connected() method that can tell us wether the TCP
 	// connection is up, and thus if we should trigger connect/disconnect.
@@ -88,7 +91,7 @@ type IRCNotifier struct {
 	BackoffCounter    Delayer
 }
 
-func NewIRCNotifier(config *Config, alertMsgs chan AlertMsg) (*IRCNotifier, error) {
+func NewIRCNotifier(ctx context.Context, stopWg *sync.WaitGroup, config *Config, alertMsgs chan AlertMsg) (*IRCNotifier, error) {
 
 	ircConfig := irc.NewConfig(config.IRCNick)
 	ircConfig.Me.Ident = config.IRCNick
@@ -113,9 +116,9 @@ func NewIRCNotifier(config *Config, alertMsgs chan AlertMsg) (*IRCNotifier, erro
 		Nick:              config.IRCNick,
 		NickPassword:      config.IRCNickPass,
 		Client:            irc.Client(ircConfig),
-		StopRunning:       make(chan bool),
-		StoppedRunning:    make(chan bool),
 		AlertMsgs:         alertMsgs,
+		ctx:               ctx,
+		stopWg:            stopWg,
 		sessionUpSignal:   make(chan bool),
 		sessionDownSignal: make(chan bool),
 		PreJoinChannels:   config.IRCChannels,
@@ -161,7 +164,9 @@ func (notifier *IRCNotifier) HandleKick(nick string, channel string) {
 	}
 	log.Printf("Being kicked out of %s, re-joining", channel)
 	go func() {
-		state.BackoffCounter.Delay()
+		if ok := state.BackoffCounter.DelayContext(notifier.ctx); !ok {
+			return
+		}
 		notifier.Client.Join(state.Channel.Name, state.Channel.Password)
 	}()
 
@@ -233,43 +238,7 @@ func (notifier *IRCNotifier) MaybeSendAlertMsg(alertMsg *AlertMsg) {
 	ircSentMsgs.WithLabelValues(alertMsg.Channel).Inc()
 }
 
-func (notifier *IRCNotifier) Run() {
-	keepGoing := true
-	for keepGoing {
-		if !notifier.Client.Connected() {
-			log.Printf("Connecting to IRC %s", notifier.Client.Config().Server)
-			notifier.BackoffCounter.Delay()
-			if err := notifier.Client.Connect(); err != nil {
-				log.Printf("Could not connect to IRC: %s", err)
-				select {
-				case <-notifier.StopRunning:
-					log.Printf("IRC routine not connected but asked to terminate")
-					keepGoing = false
-				default:
-				}
-				continue
-			}
-			log.Printf("Connected to IRC server, waiting to establish session")
-		}
-
-		select {
-		case alertMsg := <-notifier.AlertMsgs:
-			notifier.MaybeSendAlertMsg(&alertMsg)
-		case <-notifier.sessionUpSignal:
-			notifier.sessionUp = true
-			notifier.MaybeIdentifyNick()
-			notifier.JoinChannels()
-			ircConnectedGauge.Set(1)
-		case <-notifier.sessionDownSignal:
-			notifier.sessionUp = false
-			notifier.CleanupChannels()
-			notifier.Client.Quit("see ya")
-			ircConnectedGauge.Set(0)
-		case <-notifier.StopRunning:
-			log.Printf("IRC routine asked to terminate")
-			keepGoing = false
-		}
-	}
+func (notifier *IRCNotifier) ShutdownPhase() {
 	if notifier.Client.Connected() {
 		log.Printf("IRC client connected, quitting")
 		notifier.Client.Quit("see ya")
@@ -283,5 +252,56 @@ func (notifier *IRCNotifier) Run() {
 			}
 		}
 	}
-	notifier.StoppedRunning <- true
+}
+
+func (notifier *IRCNotifier) ConnectedPhase() {
+	select {
+	case alertMsg := <-notifier.AlertMsgs:
+		notifier.MaybeSendAlertMsg(&alertMsg)
+	case <-notifier.sessionDownSignal:
+		notifier.sessionUp = false
+		notifier.CleanupChannels()
+		notifier.Client.Quit("see ya")
+		ircConnectedGauge.Set(0)
+	case <-notifier.ctx.Done():
+		log.Printf("IRC routine asked to terminate")
+	}
+}
+
+func (notifier *IRCNotifier) SetupPhase() {
+	if !notifier.Client.Connected() {
+		log.Printf("Connecting to IRC %s", notifier.Client.Config().Server)
+		if ok := notifier.BackoffCounter.DelayContext(notifier.ctx); !ok {
+			return
+		}
+		if err := notifier.Client.ConnectContext(notifier.ctx); err != nil {
+			log.Printf("Could not connect to IRC: %s", err)
+			return
+		}
+		log.Printf("Connected to IRC server, waiting to establish session")
+	}
+	select {
+	case <-notifier.sessionUpSignal:
+		notifier.sessionUp = true
+		notifier.MaybeIdentifyNick()
+		notifier.JoinChannels()
+		ircConnectedGauge.Set(1)
+	case <-notifier.sessionDownSignal:
+		log.Printf("Receiving a session down before the session is up, this is odd")
+	case <-notifier.ctx.Done():
+		log.Printf("IRC routine asked to terminate")
+	}
+}
+
+func (notifier *IRCNotifier) Run() {
+	defer notifier.stopWg.Done()
+
+	for notifier.ctx.Err() != context.Canceled {
+		if !notifier.sessionUp {
+			notifier.SetupPhase()
+		} else {
+			notifier.ConnectedPhase()
+		}
+	}
+	notifier.ShutdownPhase()
 }
