@@ -16,6 +16,7 @@ package main
 
 import (
 	"bufio"
+	"context"
 	"fmt"
 	"io"
 	"log"
@@ -195,10 +196,22 @@ func makeTestServer(t *testing.T) (*testServer, int) {
 }
 
 type FakeDelayer struct {
+	DelayOnChan bool
+	StopDelay   chan bool
 }
 
 func (f *FakeDelayer) Delay() {
+	f.DelayContext(context.Background())
+}
+
+func (f *FakeDelayer) DelayContext(ctx context.Context) bool {
 	log.Printf("Faking Backoff")
+	if f.DelayOnChan {
+		log.Printf("Waiting StopDelay signal")
+		<-f.StopDelay
+		log.Printf("Received StopDelay signal")
+	}
+	return true
 }
 
 func makeTestIRCConfig(IRCPort int) *Config {
@@ -217,22 +230,28 @@ func makeTestIRCConfig(IRCPort int) *Config {
 	}
 }
 
-func makeTestNotifier(t *testing.T, config *Config) (*IRCNotifier, chan AlertMsg) {
+func makeTestNotifier(t *testing.T, config *Config) (*IRCNotifier, chan AlertMsg, context.CancelFunc, *sync.WaitGroup) {
 	alertMsgs := make(chan AlertMsg)
-	notifier, err := NewIRCNotifier(config, alertMsgs)
+	ctx, cancel := context.WithCancel(context.Background())
+	stopWg := sync.WaitGroup{}
+	stopWg.Add(1)
+	notifier, err := NewIRCNotifier(ctx, &stopWg, config, alertMsgs)
 	if err != nil {
 		t.Fatal(fmt.Sprintf("Could not create IRC notifier: %s", err))
 	}
 	notifier.Client.Config().Flood = true
-	notifier.BackoffCounter = &FakeDelayer{}
+	notifier.BackoffCounter = &FakeDelayer{
+		DelayOnChan: false,
+		StopDelay:   make(chan bool),
+	}
 
-	return notifier, alertMsgs
+	return notifier, alertMsgs, cancel, &stopWg
 }
 
 func TestPreJoinChannels(t *testing.T) {
 	server, port := makeTestServer(t)
 	config := makeTestIRCConfig(port)
-	notifier, _ := makeTestNotifier(t, config)
+	notifier, _, cancel, _ := makeTestNotifier(t, config)
 
 	var testStep sync.WaitGroup
 
@@ -250,7 +269,7 @@ func TestPreJoinChannels(t *testing.T) {
 
 	testStep.Wait()
 
-	notifier.StopRunning <- true
+	cancel()
 	server.Stop()
 
 	expectedCommands := []string{
@@ -271,7 +290,7 @@ func TestServerPassword(t *testing.T) {
 	server, port := makeTestServer(t)
 	config := makeTestIRCConfig(port)
 	config.IRCHostPass = "hostsecret"
-	notifier, _ := makeTestNotifier(t, config)
+	notifier, _, cancel, _ := makeTestNotifier(t, config)
 
 	var testStep sync.WaitGroup
 
@@ -289,7 +308,7 @@ func TestServerPassword(t *testing.T) {
 
 	testStep.Wait()
 
-	notifier.StopRunning <- true
+	cancel()
 	server.Stop()
 
 	expectedCommands := []string{
@@ -310,7 +329,7 @@ func TestServerPassword(t *testing.T) {
 func TestSendAlertOnPreJoinedChannel(t *testing.T) {
 	server, port := makeTestServer(t)
 	config := makeTestIRCConfig(port)
-	notifier, alertMsgs := makeTestNotifier(t, config)
+	notifier, alertMsgs, cancel, _ := makeTestNotifier(t, config)
 
 	var testStep sync.WaitGroup
 
@@ -345,7 +364,7 @@ func TestSendAlertOnPreJoinedChannel(t *testing.T) {
 
 	testStep.Wait()
 
-	notifier.StopRunning <- true
+	cancel()
 	server.Stop()
 
 	expectedCommands := []string{
@@ -367,7 +386,7 @@ func TestUsePrivmsgToSendAlertOnPreJoinedChannel(t *testing.T) {
 	server, port := makeTestServer(t)
 	config := makeTestIRCConfig(port)
 	config.UsePrivmsg = true
-	notifier, alertMsgs := makeTestNotifier(t, config)
+	notifier, alertMsgs, cancel, _ := makeTestNotifier(t, config)
 
 	var testStep sync.WaitGroup
 
@@ -402,7 +421,7 @@ func TestUsePrivmsgToSendAlertOnPreJoinedChannel(t *testing.T) {
 
 	testStep.Wait()
 
-	notifier.StopRunning <- true
+	cancel()
 	server.Stop()
 
 	expectedCommands := []string{
@@ -423,7 +442,7 @@ func TestUsePrivmsgToSendAlertOnPreJoinedChannel(t *testing.T) {
 func TestSendAlertAndJoinChannel(t *testing.T) {
 	server, port := makeTestServer(t)
 	config := makeTestIRCConfig(port)
-	notifier, alertMsgs := makeTestNotifier(t, config)
+	notifier, alertMsgs, cancel, _ := makeTestNotifier(t, config)
 
 	var testStep sync.WaitGroup
 
@@ -459,7 +478,7 @@ func TestSendAlertAndJoinChannel(t *testing.T) {
 
 	testStep.Wait()
 
-	notifier.StopRunning <- true
+	cancel()
 	server.Stop()
 
 	expectedCommands := []string{
@@ -482,7 +501,7 @@ func TestSendAlertAndJoinChannel(t *testing.T) {
 func TestSendAlertDisconnected(t *testing.T) {
 	server, port := makeTestServer(t)
 	config := makeTestIRCConfig(port)
-	notifier, alertMsgs := makeTestNotifier(t, config)
+	notifier, alertMsgs, cancel, _ := makeTestNotifier(t, config)
 
 	var testStep, holdUserStep sync.WaitGroup
 
@@ -504,7 +523,12 @@ func TestSendAlertDisconnected(t *testing.T) {
 
 	go notifier.Run()
 
-	alertMsgs <- AlertMsg{Channel: testChannel, Alert: disconnectedTestMessage}
+	// Alert channels is not consumed while disconnected
+	select {
+	case alertMsgs <- AlertMsg{Channel: testChannel, Alert: disconnectedTestMessage}:
+		t.Error("Alert consumed while disconnected")
+	default:
+	}
 
 	testStep.Done()
 	holdUserStep.Wait()
@@ -535,7 +559,7 @@ func TestSendAlertDisconnected(t *testing.T) {
 
 	testStep.Wait()
 
-	notifier.StopRunning <- true
+	cancel()
 	server.Stop()
 
 	expectedCommands := []string{
@@ -557,7 +581,7 @@ func TestSendAlertDisconnected(t *testing.T) {
 func TestReconnect(t *testing.T) {
 	server, port := makeTestServer(t)
 	config := makeTestIRCConfig(port)
-	notifier, _ := makeTestNotifier(t, config)
+	notifier, _, cancel, _ := makeTestNotifier(t, config)
 
 	var testStep sync.WaitGroup
 
@@ -583,7 +607,7 @@ func TestReconnect(t *testing.T) {
 	// Wait again until the last pre-joined channel is seen.
 	testStep.Wait()
 
-	notifier.StopRunning <- true
+	cancel()
 	server.Stop()
 
 	expectedCommands := []string{
@@ -613,7 +637,12 @@ func TestConnectErrorRetry(t *testing.T) {
 	// Attempt SSL handshake. The server does not support it, resulting in
 	// a connection error.
 	config.IRCUseSSL = true
-	notifier, _ := makeTestNotifier(t, config)
+	notifier, _, cancel, _ := makeTestNotifier(t, config)
+	// Pilot reconnect attempts via backoff delay to prevent race
+	// conditions in the test while we change the components behavior on
+	// the fly.
+	delayer := notifier.BackoffCounter.(*FakeDelayer)
+	delayer.DelayOnChan = true
 
 	var testStep, joinStep sync.WaitGroup
 
@@ -626,6 +655,8 @@ func TestConnectErrorRetry(t *testing.T) {
 
 	go notifier.Run()
 
+	delayer.StopDelay <- true
+
 	testStep.Wait()
 
 	// We have caused a connection failure, now check for a reconnection
@@ -641,9 +672,11 @@ func TestConnectErrorRetry(t *testing.T) {
 	server.SetHandler("JOIN", joinHandler)
 	server.SetCloseEarly(nil)
 
+	delayer.StopDelay <- true
+
 	joinStep.Wait()
 
-	notifier.StopRunning <- true
+	cancel()
 	server.Stop()
 
 	expectedCommands := []string{
@@ -664,7 +697,7 @@ func TestIdentify(t *testing.T) {
 	server, port := makeTestServer(t)
 	config := makeTestIRCConfig(port)
 	config.IRCNickPass = "nickpassword"
-	notifier, _ := makeTestNotifier(t, config)
+	notifier, _, cancel, _ := makeTestNotifier(t, config)
 	notifier.NickservDelayWait = 0 * time.Second
 
 	var testStep sync.WaitGroup
@@ -685,7 +718,7 @@ func TestIdentify(t *testing.T) {
 
 	testStep.Wait()
 
-	notifier.StopRunning <- true
+	cancel()
 	server.Stop()
 
 	expectedCommands := []string{
@@ -707,7 +740,7 @@ func TestGhostAndIdentify(t *testing.T) {
 	server, port := makeTestServer(t)
 	config := makeTestIRCConfig(port)
 	config.IRCNickPass = "nickpassword"
-	notifier, _ := makeTestNotifier(t, config)
+	notifier, _, cancel, _ := makeTestNotifier(t, config)
 	notifier.NickservDelayWait = 0 * time.Second
 
 	var testStep, usedNick, unregisteredNickHandler sync.WaitGroup
@@ -746,7 +779,7 @@ func TestGhostAndIdentify(t *testing.T) {
 
 	testStep.Wait()
 
-	notifier.StopRunning <- true
+	cancel()
 	server.Stop()
 
 	expectedCommands := []string{
@@ -770,7 +803,7 @@ func TestGhostAndIdentify(t *testing.T) {
 func TestStopRunningWhenHalfConnected(t *testing.T) {
 	server, port := makeTestServer(t)
 	config := makeTestIRCConfig(port)
-	notifier, _ := makeTestNotifier(t, config)
+	notifier, _, cancel, stopWg := makeTestNotifier(t, config)
 
 	var testStep, holdQuitWait sync.WaitGroup
 
@@ -797,9 +830,9 @@ func TestStopRunningWhenHalfConnected(t *testing.T) {
 
 	testStep.Wait()
 
-	notifier.StopRunning <- true
+	cancel()
 
-	<-notifier.StoppedRunning
+	stopWg.Wait()
 
 	holdQuitWait.Wait()
 
